/**
 * File: lex.cpp
 * -------------
 * Presents a trie-backed implementation of a Lexicon
 * that supports add, remove, contains, containsPrefix.
 */

#include "lex.h"
using namespace std;

struct node {
    bool isWord;
    Map<char, node *> suffixes;
    
    node();
    ~node();
};

node::node() {
    isWord = false;
}

node::~node() {
    for (char ch: suffixes) {
        delete suffixes[ch];
    }
}

Lex::Lex() {
    root = NULL;
}

Lex::~Lex() {
    delete root;
}

void Lex::add(const string& word) {
    ensureNodeExists(word)->isWord = true;
}

bool Lex::containsPrefix(const string& prefix) const {
    return findNode(prefix) != NULL;
}

bool Lex::contains(const string& word) const {
    const node *found = findNode(word);
    return found != NULL && found->isWord;
}

const node *Lex::findNode(const string& str) const {
    const node *curr = root;
    for (size_t pos = 0; pos < str.size() && curr != NULL; pos++) {
        curr = curr->suffixes.containsKey(str[pos]) ?
               curr->suffixes.get(str[pos]) : NULL;
    }
    
    return curr;
}

node *Lex::ensureNodeExists(const string& str) {
    node **currp = &root;
    size_t pos = 0;
    while (true) {
        if (*currp == NULL) *currp = new node;
        if (pos == str.size()) break;
        currp = &((*currp)->suffixes[str[pos]]);
        pos++;
    }
    return *currp;
}
