/**
 * File: lex.h
 * -----------
 * Defines a Lex class that emulates the Lexicon
 * class exported by the Stanford CS106 libraries.  Because
 * the CS106 library set already includes a "lexicon.h" file,
 * we shouldn't name this file that; I go with "lex.h" instead.
 */

#pragma once
#include <string>
#include "map.h"

/**
 * Class: Lex
 * ----------
 * Abstraction that closely imitiates the Lexicon abstraction you're
 * familiar with from Word Ladders and Boggle.
 */
class Lex {
public:

    /**
     * Constructor: Lex
     * ----------------
     * Constructs an empty Lex object.
     */
    Lex();

    /**
     * Destructor: ~Lex
     * ----------------
     * Disposes of any and all resources that have accumulated over
     * the lifetime of the Lex instance.
     */
    ~Lex();
    
    /**
     * Method: add
     * -----------
     * Adds the supplied word to the lexicon if it isn't
     * already present.
     */
    void add(const std::string& word);

    /**
     * Method: contains
     * ----------------
     * Returns true if and only if the receiving lexicon contains
     * the supplied word.
     */
    bool contains(const std::string& word) const;

    /**
     * Method: containsPrefix
     * ----------------------
     * Returns true if and only if the supplied string is a prefix
     * of one or more words present in the lexicon.  So, if
     * the only word in the lexicon is "art", then "", "a", "ar", and
     * "art" are all valid prefixes.
     */
    bool containsPrefix(const std::string& prefix) const;
    
private:
    struct node *root;
    const struct node *findNode(const std::string& str) const;
    struct node *ensureNodeExists(const std::string& str);
};
